#!/usr/bin/env python

#
# Copyright 2025 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Data structure for tables.  A way to represent a table
without any specifics on output formatting.
'''

# pylint: disable=R0903


class Table():
    '''
    Table: The root of all rendered table classes

    Design Note
    -----------
    Most of the data assignment code is handled here.
    The rendering code is handled by the subclasses.

    No output should actually be generated until the
    render() call is made.
    '''
    def __init__(self):
        self._header = None
        self._footer = None
        self.bchunks = []
        self.col_specs = []
        self.classes = []
        self.id = ''

    def set_classes(self, in_classes):
        '''
        Set the class types for the table.
        '''
        self.classes = in_classes

    def set_id(self, in_id):
        '''
        Set an id value for the table.
        '''
        self.id = in_id

    def add_column(self, in_column_spec):
        '''
        Add a column specification to the table.
        '''
        in_column_spec.set_column_number(len(self.col_specs)+1)
        self.col_specs.append(in_column_spec)

    def set_columns(self, *in_cols):
        '''
        Setup the columns of the table.
        '''
        # self.col_specs = in_cols
        self.col_specs = []
        for col_i in in_cols:
            if isinstance(col_i, TableColumnSpec):
                self.col_specs.append(col_i)
            else:
                raise TableSetupException('Invalid column spec object')
        self.col_specs[-1].end_of_row = True
        self.col_specs[-1].build_py_format()

    def add_body(self, in_header=None):
        '''
        Add a data structure that holds a bunch of table rows.
        In HTML parlance, this is the equivelant of a TBODY element.
        '''
        self.bchunks.append(TableBodyChunk(self))
        self.bchunks[-1].set_body_header(in_header)

    def set_body_header(self, in_header=None):
        '''
        Set the header for a body chunk, if there is one.
        '''
        if len(self.bchunks) > 0:
            self.bchunks[-1].set_body_header(in_header)
        else:
            self.bchunks.append(TableBodyChunk(self))
            self.bchunks[-1].set_body_header(in_header)

    def add_row(self, *in_values):
        '''
        Add a row to the current body chunk.
        '''
        if len(self.bchunks) > 0:
            self.bchunks[-1].add_row(in_values)
        else:
            self.bchunks.append(TableBodyChunk(self))
            self.bchunks[-1].add_row(in_values)


class TableBodyChunk():
    '''
    A collection of rows in the main body
    of the table.

    A table will have at least one of these
    objects.
    '''
    def __init__(self, in_table):
        self.rows = []
        self.body_header = None
        self.table = in_table

    def add_row(self, in_values):
        '''
        Add a row to the body chunk.
        '''
        self.rows.append(TableBodyRow(self, in_values))

    def set_body_header(self, in_header):
        '''
        Set the optional header of the body chunk.
        '''
        self.body_header = in_header


class TableColumnAlign():
    '''
    Constants for table text justification.
    '''
    LEFT = 1
    RIGHT = 2
    CENTER = 3


class TableColumnSpec():
    '''
    Specifications for a table column.  There is one
    instance of this class for every column of the table.
    '''
    def __init__(self, in_hdr_text='', in_width=20,
                 in_align=TableColumnAlign.LEFT):
        self.py_format = ''
        self.hdr_text = in_hdr_text
        self.width = in_width
        self.align = in_align
        self.end_of_row = False
        self.col_number = -1
        self.build_py_format()

    def set_column_number(self, in_co_no):
        '''
        Set the column number value.
        '''
        self.col_number = in_co_no

    def build_py_format(self):
        '''
        Setup a format string for the cell value.
        '''
        if self.align == TableColumnAlign.RIGHT:
            self.py_format = ">" + str(self.width)
        else:
            if self.end_of_row:
                self.py_format = ''
            else:
                self.py_format = str(self.width)


class TableBodyRow():
    '''
    Represents a single row in a table body.
    '''
    def __init__(self, in_chunk, in_values):
        self.cells = []
        self.body_chunk = in_chunk
        col_specs = self.body_chunk.table.col_specs
        col_iter = 0
        for val in in_values:
            self.cells.append(TableDataCell(val, col_specs[col_iter]))
            col_iter += 1


class TableHeaderCell():
    '''
    A single cell designed to contain header data.
    '''
    def __init__(self, in_value, in_spec):
        self.value = in_value
        self.spec = in_spec


class TableDataCell():
    '''
    A single cell, containing data.
    '''
    def __init__(self, in_value, in_spec):
        self.value = in_value
        self.spec = in_spec


class TableHeaderColumnSpec(TableColumnSpec):
    '''
    Class for a table column of sub-headers.

    It is essentially identical to the table column class,
    but with a different name so it can be differentiated.
    '''


class TableColFlags():
    '''
    Special option flags that apply to a column in a table.

    HEADER_COLUMN shows a column actually contains headers,
    and not values.

    '''
    HEADER_COLUMN = 1


class TableFlags():
    '''
    Special option flags that apply to a table.
    '''
    VERTICAL_HEADERS = 1


class TableSetupException(Exception):
    '''
    The TableSetupException class is for situation where the
    setup of the table isn't complete, or is missing
    data that makes it impossible to render.
    '''
    def __init__(self, in_message):
        super().__init__(in_message)
        self.message = in_message

    def __str__(self):
        return self.message



#!/usr/bin/env python

#
# Copyright 2025 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Album objects
'''

# pylint: disable=too-few-public-methods
# pylint: disable=too-many-instance-attributes
# pylint: disable=R0801


from datetime import timedelta
from media.data.media.contents.internal import AbstractContent
from media.data.media.contents.generic.catalog import Title
from media.data.media.contents.audio.album.catalog import AlbumCatalog
from media.data.media.contents.audio.album.classification \
    import AlbumClassification
from media.data.media.contents.audio.album.technical \
    import AlbumTechnical
from media.data.media.contents.audio.elements.song import Song
from media.data.media.contents.audio.elements.dialogue import Dialogue
from media.general.sorting.index import ContentIndex
from media.xml.namespaces import Namespaces


class Album(AbstractContent):
    '''
    Album class
    '''
    def __init__(self, in_element):
        super().__init__()
        self.title = ''
        self.catalog = None
        self.classification = None
        self.technical = None
        self.elements = []
        self.runtime = timedelta(seconds=0)
        self.s_index = None
        self._process(in_element)

    def _process(self, in_element):
        '''
        Processing of top level album elements.
        '''
        for child in in_element:
            tagname = Namespaces.ns_strip(child.tag)
            if tagname == 'title':
                self.title = Title(child.text)
            elif tagname == 'catalog':
                self.catalog = AlbumCatalog(child)
            elif tagname == 'classification':
                self.classification = AlbumClassification(child)
            elif tagname == 'elements':
                self._load_elements(child)
        self._post_load_process()

    def _load_elements(self, in_element):
        '''
        Load element objects and pack them in the elements array.
        '''
        for child in in_element:
            tagname = Namespaces.ns_strip(child.tag)
            if tagname == 'song':
                self.elements.append(Song(child, self))
            elif tagname == 'dialogue':
                self.elements.append(Dialogue(child, self))

    def _post_load_process(self):
        super()._post_load_process()
        self.technical = AlbumTechnical(self)
        self.s_index = AlbumIndexEntry(self)


class AlbumIndexEntry(ContentIndex):
    '''
    Index object for sorting albums.
    '''
    def __init__(self, in_album):
        super().__init__()
        self.album = in_album
        self.year = 0
        self.artists = None
        self.decade = 0
        self.primary_g = None
        self.first_letter = None
        self.runtime = None
        self._extract_fields()

    def _extract_fields(self):
        '''
        Extract the title, the artist(s), the copyright year,
        and the genre.  If there's no genre, but there is
        a soundtrack or score element, then create a pseudo-genre
        called Soundtrack or Score respestively.
        '''
        if self.album.catalog:
            cat = self.album.catalog
            if cat.copyright:
                self.year = cat.copyright.year
                self.decade = int(self.year / 10)
            if cat.artists:
                self.artists = AlbumIndexArtists(cat.artists)
        if self.album.classification:
            cls = self.album.classification
            if cls.genres:
                if cls.genres.primary:
                    self.primary_g = cls.genres.primary
            if cls.soundtrack:
                self.primary_g = 'Soundtrack'
            if cls.score:
                self.primary_g = 'Score'
        if not self.artists:
            # It's unlikely we will ever reach this point.
            self.artists = AlbumIndexArtists()
        self.runtime = self.album.technical.runtime
        self.sort_title = self.album.sort_title
        self.first_letter = self.sort_title[0]


class AlbumIndexArtists():
    '''
    An object that works to manage one or more
    artist names with a mechanism for sorting
    and presentation.
    '''
    def __init__(self, in_artists=None):
        self.artists = []
        self.sort_string = ''
        self.formal_string = ''
        if in_artists:
            self.artists = in_artists
            self._build_data()

    def _build_data(self):
        '''
        Build strings suitable for sorting
        the album artists, and also
        presenting the album artists.
        '''
        sort_str = ''
        form_str = ''
        if self.artists:
            for art in self.artists:
                sort_str += art.sort_value + '_'
                form_str += str(art) + ', '
            self.sort_string = sort_str[:-1]
            self.formal_string = form_str[:-2]
        else:
            self.sort_string = 'unknown'
            self.formal_string = 'Unknown'

    def __lt__(self, other):
        return self.sort_string < other.sort_string

    def __rt__(self, other):
        return self.sort_string > other.sort_string

    def __eq__(self, other):
        return self.sort_string == other.sort_string

    def __str__(self):
        return self.formal_string

    def __hash__(self):
        '''
        We need to generate the hash value ourselves.
        '''
        return hash(self.sort_string + self.formal_string)



#!/usr/bin/env python

#
# Copyright 2023 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#


'''
generic validation module
'''

# pylint: disable=R0801
# pylint: disable=W0613

from datetime import timedelta
from media.validation.core.faults import Fault, FaultLevel
from media.validation.core.status import StatusCode, TestResult
from media.validation.core.tests import AbstractValidator

DEFMAXSCORE = 5


class MovieTechnicalValidator(AbstractValidator):
    '''
    Technical validation block for all movies.
    '''
    def __init__(self):
        super().__init__()
        self.load_objects()

    def load_objects(self):
        '''
        Load additional class objects with validation tests.
        '''
        self.objects.append(MovieRuntimeValidator())

    def run_standard_tests(self, in_dish, in_level=5):
        '''
        Run the standard test suite.
        '''
        for tobj in self.objects:
            tobj.run_standard_tests(in_dish, in_level)


class MovieRuntimeValidator(AbstractValidator):
    '''
    Tests against the runtime value of all A/V
    content objects.
    '''

    def run_standard_tests(self, in_dish, in_level=5):
        '''
        Run the standard tests for movie runtime values.
        '''
        self.test_runtime_value(in_dish)

    def test_runtime_value(self, in_dish):
        '''
        Test that the overall runtime value makes sense.
        '''
        f_text = 'Movie runtimes are typically more than 45 minutes'
        test_r = TestResult('movie.technical.runtime.value', DEFMAXSCORE)
        technical = in_dish.item.technical
        if technical is not None:
            runtime = technical.runtime
            if runtime:
                runtime_value = runtime.overall or timedelta(0)
                movie_minimum = timedelta(minutes=45)
                if runtime_value <= movie_minimum:
                    fault = Fault(FaultLevel.NOTICE, f_text)
                    test_r.result(3, StatusCode.FAIL, fault)
                else:
                    test_r.autopass()
            else:
                test_r.skip()
        else:
            test_r.skip()
        in_dish.add_result(test_r)



#!/usr/bin/env python

#
# Copyright 2025 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Module to report on all named within a movie
'''

# pylint: disable=R0801

import os
import argparse
from media.general.sorting.organizer import Organizer
from media.tools.common import load_movies


class NameJobTitleMap():
    '''Crewname object that lists every job role in every title.
    '''
    def __init__(self, in_name, in_job, in_title):
        self.name = in_name
        self.jobs = {}
        self.jobs[in_job] = [in_title]

    def add_job_title(self, in_job, in_title):
        '''
        Add a job role and movie title to the name.
        '''
        if in_job in self.jobs:
            self.jobs[in_job].append(in_title)
        else:
            self.jobs[in_job] = [in_title]

    @classmethod
    def header(cls):
        '''
        Output a simple header.
        '''
        out = f"{'Family Name':20s} {'Given Name':15s} " + \
              f"{'Job':20s} {'Title':25s}\n" + \
              f"{'=' * 20} {'=' * 15} {'=' * 20} {'=' * 25}"
        return out

    def __str__(self):
        out = ''
        name_l = 1
        for job_i in sorted(self.jobs):
            job_l = 1
            for title_i in sorted(self.jobs[job_i]):
                if name_l == 1:
                    out = f"{self.name.family:20s} {self.name.given:15s} " + \
                          f"{job_i:20s} {title_i!s:25s}\n"
                else:
                    if job_l == 1:
                        out += f"{' ':36s} {job_i:20s} {title_i!s:25s}\n"
                    else:
                        out += f"{' ':57s} {title_i!s:25s}\n"
                job_l += 1
                name_l += 1
        return out

    def __lt__(self, other):
        return self.name < other.name

    def __gt__(self, other):
        return self.name > other.name

    def __eq__(self, other):
        return self.name == other.name


def extract_from_list(in_dict, in_crew_job, in_job_title, in_movie_title):
    '''
    Extract the name of a crew member from a job array.
    '''
    for name_i in in_crew_job:
        if name_i in in_dict:
            in_dict[name_i].add_job_title(in_job_title, in_movie_title)
        else:
            in_dict[name_i] = NameJobTitleMap(name_i, in_job_title,
                                              in_movie_title)


def extract_role_from_list(in_dict, in_crew_roles,
                           in_job_title, in_movie_title):
    '''
    Extract the name of the actor from a role object.
    '''
    for name_i in in_crew_roles:
        if name_i.actor in in_dict:
            in_dict[name_i.actor].add_job_title(in_job_title, in_movie_title)
        else:
            in_dict[name_i.actor] = NameJobTitleMap(name_i.actor,
                                                    in_job_title,
                                                    in_movie_title)


def grab_crew_names(movies):
    '''
    Extract name objects from the arrays specific to
    the job functions.
    '''
    nm_dict = {}
    for movie in movies:
        if movie.crew is not None:
            if movie.crew.directors:
                extract_from_list(nm_dict, movie.crew.directors,
                                  'Director', movie.title)
            if movie.crew.writers:
                extract_from_list(nm_dict, movie.crew.writers,
                                  'Writer', movie.title)
            if movie.crew.cinemap:
                extract_from_list(nm_dict, movie.crew.cinemap,
                                  'Cinemaphotographer', movie.title)
            if movie.crew.editors:
                extract_from_list(nm_dict, movie.crew.editors,
                                  'Editor', movie.title)
            if movie.crew.cast:
                extract_role_from_list(nm_dict, movie.crew.cast.cast,
                                       'Cast', movie.title)
    return list(nm_dict.values())


def list_names(in_names):
    '''
    Generate output from the list of passed names.
    '''
    print(NameJobTitleMap.header())
    for name_i in sorted(in_names):
        print(name_i, end='')


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Simple movie list.')
    parser.add_argument('--mediapath', help='path of media library')
    parser.add_argument('--random', type=int, help='show X names')
    args = parser.parse_args()
    mediapath = args.mediapath or os.environ['MEDIAPATH']
    if not mediapath:
        parser.print_help()
    all_movies = load_movies(mediapath)
    all_names = grab_crew_names(all_movies)
    if args.random:
        rand_limit = args.random
        list_names(Organizer.get_random_sample(all_names, rand_limit))
    else:
        list_names(all_names)



#!/usr/bin/env python

#
# Copyright 2023 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Objects pertaining to validation test faults.
'''

# pylint: disable=R0903


class FaultLevel():
    '''
    Fault levels.
    '''
    NOTICE = 1
    WARNING = 2
    CRITICAL = 3


class Fault():
    '''
    A fault is the equivelant of an exception
    when validating media records.
    '''
    def __init__(self, in_level, in_message, in_text=None):
        self.level = in_level
        self.message = in_message
        self.text = in_text

    def lvl_str(self):
        '''
        Return the fault level as a string.
        '''
        matrix = {
                  FaultLevel.NOTICE: 'NOTICE',
                  FaultLevel.WARNING: 'WARNING',
                  FaultLevel.CRITICAL: 'CRITICAL'
                 }
        return matrix[self.level]

    def __str__(self):
        if self.text:
            return f"{self.lvl_str()}: {self.message} ({self.text})"
        return f"{self.lvl_str()}: {self.message}"



#!/usr/bin/env python

#
# Copyright 2025 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Run simple validation tests against the movies, report any incomplete data.
'''

# pylint: disable=R0801
# pylint: disable=too-few-public-methods

import os
import argparse
from datetime import timedelta
from media.general.sorting.organizer import Organizer
from media.tools.common import load_movies


class Status():
    '''
    Status object for a given movie.

    It contains the movie object, an array of faults,
    and a simple flag determining whether or not the
    film is acceptable.
    '''
    def __init__(self, in_movie):
        self.movie = in_movie
        self.acceptable = True
        self.faults = []

    def add_fault(self, in_fault):
        '''Add a fault object to the movie status'''
        self.faults.append(in_fault)
        self.acceptable = False

    def report_faults(self):
        '''Report faults for a movie if present.'''
        out = ''
        if not self.acceptable:
            fault1 = self.faults.pop(0)
            out = f"{self.movie.title!s:50s} {fault1!s}\n"
            for flt in self.faults:
                out += f"{' ' * 50} {flt!s}\n"
        return out

    @classmethod
    def status_header(cls):
        '''Return a generic output header.'''
        return f"{'Movie Title':50s} {'Faults'}\n" + \
               f"{'-' * 50:s} {'-' * 25:s}"

    def __lt__(self, other):
        return self.movie.unique_key < other.movie.unique_key

    def __gt__(self, other):
        return self.movie.unique_key > other.movie.unique_key


class Fault():
    '''
    Fault class, identifying the reason for the fault, and
    a severity score.
    '''
    def __init__(self, in_reason, in_score=5):
        self.reason = in_reason
        self.score = in_score

    def __str__(self):
        return self.reason


class Tester():
    '''
    Simple tester class that examines the attributes of a movie,
    and identifies faults if they are present.
    '''

    @classmethod
    def test_plot(cls, movie_status):
        '''Test the movie plot.'''
        p_string = str(movie_status.movie.story.plot).strip()
        if not p_string:
            movie_status.add_fault(Fault("Plot missing"))
        else:
            words = p_string.split()
            pwc = len(words)
            if pwc < 20:
                movie_status.add_fault(Fault("Plot has less than 20 words"))

    @classmethod
    def test_technical(cls, movie_status):
        '''Test the movie technical data.'''
        if movie_status.movie.technical:
            tech = movie_status.movie.technical
            if tech.runtime:
                runtime = tech.runtime.overall or timedelta(0)
                if runtime == timedelta(0):
                    movie_status.add_fault(Fault("Runtime is undefined"))
                elif runtime == timedelta(seconds=1, minutes=1, hours=1):
                    movie_status.add_fault(Fault("Suspicious runtime value"))
        else:
            movie_status.add_fault(Fault("Technical block missing"))

    @classmethod
    def test_catalog(cls, movie_status):
        '''Test the movie catalog data.'''
        if movie_status.movie.catalog:
            mcat = movie_status.movie.catalog
            if mcat.copyright:
                if len(mcat.copyright.holders) == 0:
                    movie_status.add_fault(Fault("No copyright holder info"))
            else:
                movie_status.add_fault(Fault("No copyright info"))


def validate_movies(in_list):
    '''
    Initialize all the movie status objects, and then
    run through the tests.
    '''
    m_status = []
    for movie in in_list:
        m_status.append(Status(movie))
    for movie_s in m_status:
        Tester.test_plot(movie_s)
        Tester.test_technical(movie_s)
        Tester.test_catalog(movie_s)
    return m_status


def report_validation(in_list):
    '''
    Report any movie faults if they are present.
    '''
    print(Status.status_header())
    for movie_s in sorted(in_list):
        if not movie_s.acceptable:
            print(movie_s.report_faults(), end='\r')


def report_stats(all_movies_c, sample_c, in_reports):
    '''
    Provide a summary report on all movie faults.
    '''
    fault_count = 0
    for report_i in in_reports:
        if not report_i.acceptable:
            fault_count += 1
    print(f"{'-' * 50} {'-' * 25}")
    print(f"{'Movie Count :':>25s} {all_movies_c}")
    print(f"{'Random Sample Size :':>25s} {sample_c} " +
          f"({sample_c/all_movies_c*100:5.2f}%)")
    print(f"{'Fault Count :':>25s} {fault_count}")


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Simple movie list.')
    parser.add_argument('--mediapath', help='path of media library')
    parser.add_argument('--random', type=int, help='print X random entries')
    parser.add_argument('--stats', action='store_true',
                        help='report statistics')
    args = parser.parse_args()
    mediapath = args.mediapath or os.environ['MEDIAPATH']
    if not mediapath:
        parser.print_help()
    all_movies = load_movies(mediapath)
    if args.random:
        sample = Organizer.get_random_sample(all_movies, args.random)
    else:
        sample = all_movies
    status_reports = validate_movies(sample)
    report_validation(status_reports)
    if args.stats:
        report_stats(len(all_movies), len(sample), status_reports)



#!/usr/bin/env python

#
# Copyright 2022 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''Main module file for the Movie() and Title() objects'''

# pylint: disable=too-few-public-methods

from media.xml.namespaces import Namespaces


class Filing():
    '''
    Data identifying how all instances of this media are
    filed.

    Not done to the invividual instance level, but a more
    generic scope, where all physical copies may be kept in
    a curated collection.
    '''
    def __init__(self, in_element):
        self.catalog = ''
        self.collections = []
        for child in in_element:
            c_tag = Namespaces.ns_strip(child.tag)
            if c_tag == 'catalog':
                self.catalog = child.text
            if c_tag == 'collection':
                self.collections.append(child.text)



#!/usr/bin/env python

#
# Copyright 2023 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#


'''
generic catalog validation module
'''

from media.validation.core.faults import Fault, FaultLevel
from media.validation.core.status import StatusCode, TestResult
from media.validation.core.tests import AbstractValidator


class GenericCatalogValidator(AbstractValidator):
    '''
    Catalog data validator.
    '''
    def __init__(self):
        super().__init__()
        self.load_objects()

    def load_objects(self):
        '''
        Load additional test pbjects.
        '''
        self.objects.append(GenericCatalogCopyrightValidator())

    def run_standard_tests(self, in_dish, in_level=5):
        '''
        Run all tests related to the catalog element.
        '''
        self.local_tests = [self.test_catalog_present,
                            self.test_copyright_present]
        for ltst in self.local_tests:
            ltst(in_dish)
        for tst in self.objects:
            tst.run_standard_tests(in_dish, in_level)

    def test_catalog_present(self, in_dish):
        '''
        Test that the catalog element was present.
        '''
        test_r = TestResult('generic.catalog', 5)
        cata = in_dish.item.catalog
        if cata is not None:
            test_r.autopass()
        else:
            fault = Fault(FaultLevel.WARNING, 'No catalog element')
            test_r.result(0, StatusCode.FAIL, fault)
        in_dish.add_result(test_r)

    def test_copyright_present(self, in_dish):
        '''
        Test that the copyright element was present.
        '''
        test_r = TestResult('generic.catalog.copyright', 5)
        cata = in_dish.item.catalog
        if cata.copyright is not None:
            test_r.autopass()
        else:
            fault = Fault(FaultLevel.WARNING, 'No copyright element')
            test_r.result(0, StatusCode.FAIL, fault)
        in_dish.add_result(test_r)


class GenericCatalogCopyrightValidator(AbstractValidator):
    '''
    Copyright data validator.
    '''
    def run_standard_tests(self, in_dish, in_level=5):
        '''
        Run all tests related to copyright.
        '''
        self.test_copyright_year(in_dish)
        if in_level <= 4:
            self.test_copyright_holders(in_dish)

    def test_copyright_year(self, in_dish):
        '''
        Test copyright year value.
        '''
        test_r = TestResult('generic.catalog.copyright.year', 5)
        cpy = in_dish.item.catalog.copyright
        if cpy is not None:
            if cpy.year > 0:
                test_r.autopass()
            else:
                fault = Fault(FaultLevel.WARNING, 'No copyright year value')
                test_r.result(0, StatusCode.FAIL, fault)
        else:
            test_r.skip()
        in_dish.add_result(test_r)

    def test_copyright_holders(self, in_dish):
        '''
        Test copyright holder values.
        '''
        test_r = TestResult('generic.catalog.copyright.holders', 5)
        cpy = in_dish.item.catalog.copyright
        if cpy is not None:
            if len(cpy.holders) > 0:
                test_r.autopass()
            else:
                fault = Fault(FaultLevel.NOTICE, 'No copyright holders')
                test_r.result(0, StatusCode.FAIL, fault)
        else:
            test_r.skip()
        in_dish.add_result(test_r)



#!/usr/bin/env python

#
# Copyright 2024 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Standard text format reports for audio albums.
'''

# pylint: disable=R0801

from datetime import timedelta


class OneLiner():
    '''
    Format for a single line summary of an album.
    '''
    def __init__(self, in_album):
        self.album = in_album
        self.output = ""
        self.build()

    def build(self):
        """
        Construct a one-line summary of an album.
        """
        y_string = build_copyright_year(self.album)
        runtime = build_runtime(self.album)
        genre = build_genre_classification(self.album)
        artist = build_artist_string(self.album)
        self.output = f"{self.album.title!s:40s} " + \
                      f"{y_string:4s} " + \
                      f"{artist:20s} " + \
                      f"{runtime:8s} " + \
                      f"{genre:s}\n"

    def __str__(self):
        return self.output

    @classmethod
    def header_fields(cls):
        """
        Return header titles for column output.
        """
        out = f"{'Title':40s} " + \
              f"{'Year':4s} " + \
              f"{'Artist':20s} " + \
              f"{'Runtime':8s} " + \
              f"{'Genre':20s}\n" + \
              cls.header_line()
        return out

    @classmethod
    def header_line(cls):
        """
        Return a simple line to go under the header fields.
        """
        return f"{'=' * 40} {'=' * 4} {'=' * 20} {'=' * 8} {'=' * 20}\n"


def build_artist_string(in_album):
    '''
    Build a string of artist names.
    '''
    o_string = ''
    for art_o in in_album.catalog.artists:
        art_s = str(art_o)
        o_string += art_s + ", "
    o_string = o_string[:-2]
    return o_string


def build_genre_classification(in_album):
    '''
    Build a text classification string.
    '''
    o_string = ''
    classification = in_album.classification
    if classification.genres:
        if classification.genres.primary:
            o_string = f"{classification.genres.primary}"
            if classification.genres.secondary:
                o_string += '/' + '/'.join(classification.genres.secondary)
    if classification.soundtrack:
        if not o_string:
            o_string = 'Soundtrack'
    elif classification.score:
        if not o_string:
            o_string = 'Score'
    return o_string


def build_runtime(in_album):
    '''
    Construct a presentable version of the runtime value.

    The string hack to remove the microseconds came from a
    SO posting.
    '''
    runtime = timedelta(seconds=0)
    if in_album.technical.runtime:
        runtime = in_album.technical.runtime
    return str(runtime).split('.', maxsplit=1)[0]


def build_copyright_year(in_album):
    '''
    Construct a printable version of the copyright year.
    '''
    year_s = "0000"
    if in_album.catalog:
        if in_album.catalog.copyright:
            year_s = f"{in_album.catalog.copyright.year:4d}"
    return year_s



#!/usr/bin/env python

#
# Copyright 2024 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''The medium release class'''

# pylint: disable=too-few-public-methods
# pylint: disable=R0801

from media.xml.namespaces import Namespaces


class MediumDeviceMap():
    '''Static output representation on media types.'''
    f_map = {
            "audiocd": "Audio CD",
            "casette": "Casette",
            "record": "Record",
            "dvd": "DVD",
            "bluray": "Blu-Ray",
            "bluray3d": "Blu-Ray 3D",
            "hddvd": "HD-DVD",
            "ultrahd": "Ultra HD",
            "vhs": "VHS",
            "betamax": "Betamax"
        }

    @classmethod
    def formal_convert(cls, in_type):
        '''
        Return the type value in a more friendly string.
        '''
        if in_type in cls.f_map:
            return MediumDeviceMap.f_map[in_type]
        return "UNKNOWN"


class AbstractDevice():
    '''
    Abstract device class.  Identifies the type of physical media.
    '''
    def __init__(self):
        self.type_name = None
        self.formal_name = None


class BaseDevice(AbstractDevice):
    '''
    Stump class for device.  More device classes will be added later.
    '''
    def __init__(self, in_element):
        super().__init__()
        if in_element is not None:
            self._process_xml_stream(in_element)

    def _process_xml_stream(self, in_element):
        e_tag = Namespaces.ns_strip(in_element.tag)
        if e_tag in MediumDeviceMap.f_map:
            self.type_name = e_tag
            self.formal_name = MediumDeviceMap.f_map[e_tag]



#!/usr/bin/env python

#
# Copyright 2025 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''Common code related to content object sorting and grouping.'''

# pylint: disable=R0801
# pylint: disable=too-few-public-methods


from media.general.sorting.groups import (
        Grouping, GroupAll, GroupTitleAlphabetical,
        GroupDecade, GroupPrimaryGenre)
from media.general.sorting.batch import Batch
from media.general.sorting.organizer import AbstractOrganizer


class AlbumOrganizer(AbstractOrganizer):
    '''
    The AlbumOrganizer is a custom organizer sublcass for music
    albums, that allows for one additional grouping parameter,
    grouping by the name of the music artist.
    '''
    G_ARTIST = 10

    def _build_batches(self):
        if self.grouping == AbstractOrganizer.G_NONE:
            self.batches = GroupAll.group(self.working)
        elif self.grouping == AbstractOrganizer.G_ANY_ALPHA:
            self.batches = GroupTitleAlphabetical.group(self.working)
        elif self.grouping == AbstractOrganizer.G_ANY_DECADE:
            self.batches = GroupDecade.group(self.working)
        elif self.grouping == AbstractOrganizer.G_ANY_GENRE:
            self.batches = GroupPrimaryGenre.group(self.working)
        elif self.grouping == AlbumOrganizer.G_ARTIST:
            self.batches = GroupArtist.group(self.working)
        else:
            self.batches = GroupAll.group(self.working)
            print('group parameter did not match')
            # maybe we throw an exception at this point??


class GroupArtist(Grouping):
    '''
    Group entries based on the artist name.
    '''
    @classmethod
    def group(cls, entries):
        '''
        Return multiple batches, grouped by album artist.
        '''
        groups = {}
        for con_ind in entries:
            if con_ind.artists not in groups:
                groups[con_ind.artists] = Batch(con_ind.artists.sort_string,
                                                con_ind.artists.formal_string,
                                                con_ind)
            else:
                groups[con_ind.artists].append(con_ind)
        return groups



#!/usr/bin/env python

#
# Copyright 2025 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Module related to the media catalog, which should be universal
across all media types
'''

# pylint: disable=too-few-public-methods

from media.general.stringtools import build_filename_string, build_sort_string
from media.xml.namespaces import Namespaces
from media.xml.functions import xs_bool


class Title():
    '''Movie title object'''
    def __init__(self, in_title):
        self.title = in_title.strip()
        if len(self.title) == 0:
            raise TitleValueException("Invalid Title")
        self.sort_title = build_sort_string(self.title)
        self.file_title = build_filename_string(self.title)

    def __hash__(self):
        return hash(self.sort_title)

    def __lt__(self, other):
        return self.sort_title < other.sort_title

    def __gt__(self, other):
        return self.sort_title > other.sort_title

    def __eq__(self, other):
        return self.sort_title == other.sort_title

    def __str__(self):
        return self.title


class TitleValueException(Exception):
    '''Exception raised when a WOA has an invalid title.'''
    def __init__(self, message):
        super().__init__(message)
        self.message = message

    def __str__(self):
        return self.message


class AbstractCatalog():
    '''
    Abstract class for all content objects.
    Identifies references to the media, and
    pointers to the media.
    '''
    def __init__(self):
        self.copyright = None
        self.alt_titles = None
        self.unique_index = None

    def _process(self, in_element):
        for child in in_element:
            e_name = Namespaces.ns_strip(child.tag)
            if e_name == 'copyright':
                self.copyright = Copyright(child)
            elif e_name == 'altTitles':
                self.alt_titles = AlternateTitles(child)
            elif e_name == 'ucIndex':
                self.unique_index = UniqueConstraints(child)
        self._post_load_process()

    def _post_load_process(self):
        if not self.alt_titles:
            self.alt_titles = AlternateTitles(None)


class Catalog(AbstractCatalog):
    '''
    Empty catalog class.
    '''
    def __init__(self, in_element):
        super().__init__()
        if in_element is not None:
            self._process(in_element)


class Copyright():
    '''Copyright information for the given media'''
    def __init__(self, in_chunk):
        self.year = 0
        self.holders = []
        if in_chunk is not None:
            self._process(in_chunk)

    def _process(self, in_chunk):
        '''Iterate through the elements to map the data to the object'''
        for child in in_chunk:
            ele_name = Namespaces.ns_strip(child.tag)
            if ele_name == 'year':
                self.year = int(child.text)
            if ele_name == 'holders':
                self._process_copyright_holders(child)

    def _process_copyright_holders(self, in_chunk):
        for child in in_chunk:
            ele_name = Namespaces.ns_strip(child.tag)
            if ele_name == 'holder':
                self.holders.append(child.text)

    def __format__(self, format_spec):
        return f"{self.year}"


class AlternateTitles():
    '''
    All possible titles that directly reference the work of art
    '''
    def __init__(self, in_chunk):
        self.original_title = ""
        self.variant_title = None
        self.production_title = ""
        self.distribution_title = ""
        self.variant_sort = False
        self.variant_speak = False
        if in_chunk is not None:
            self._process(in_chunk)

    def _process(self, in_chunk):
        for child in in_chunk:
            e_name = Namespaces.ns_strip(child.tag)
            if e_name == 'originalTitle':
                self.original_title = child.text
            elif e_name == 'productionTitle':
                self.production_title = child.text
            elif e_name == 'distributiontitle':
                self.distribution_title = child.text
            elif e_name == 'variantTitle':
                self.variant_title = Title(child.text)
                if 'sortable' in child.attrib:
                    self.variant_sort = xs_bool(child.attrib['sortable'])
                if 'textToSpeech' in child.attrib:
                    self.variant_speak = True


class UniqueConstraints():
    '''
    Optional unique identifier.
    '''
    def __init__(self, in_chunk):
        self.index = 0
        self.note = ""
        if in_chunk is not None:
            self._process(in_chunk)

    def _process(self, in_chunk):
        for child in in_chunk:
            e_name = Namespaces.ns_strip(child.tag)
            if e_name == 'value':
                self.index = int(child.text)
            elif e_name == 'note':
                self.note = child.text



#!/usr/bin/env python

#
# Copyright 2023 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
List out all physical media, one per line.
'''

# pylint: disable=R0801

import os
import argparse
import media.data.media.meta.authorship as MA
from media.tools.common import (
        load_media_dev
        )


def list_authorship_recs(in_list):
    '''
    Provide a summary list of all movies, sorted by title or runtime.
    '''
    for record_i in in_list:
        title = record_i.title
        a_string = ''
        if record_i.catalog.authors:
            for a_name in record_i.catalog.authors:
                a_string += a_name.name + " "
            a_string = a_string[:-1]
        print(f"{title:40s} ({a_string})")
        if record_i.changelog:
            for c_rec in record_i.changelog:
                if issubclass(c_rec.__class__, MA.CreationRecord):
                    print('-' * 50)
                    print(f"Created: {c_rec.tstamp!s}")
        print()


def prep_list(in_devices):
    '''
    Build the list of output entries.
    '''
    out_l = []
    for media_i in in_devices:
        if media_i.author_record:
            out_l.append(media_i.author_record)
    return out_l


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Simple media list.')
    parser.add_argument('--mediapath', help='path of media library')
    args = parser.parse_args()

    mediapath = args.mediapath or os.environ['MEDIAPATH']
    if not mediapath:
        parser.print_help()
    devices = load_media_dev(mediapath)
    chunks = prep_list(devices)
    list_authorship_recs(chunks)



#!/usr/bin/env python

#
# Copyright 2024 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Album Classification objects
'''

# pylint: disable=too-few-public-methods

from media.xml.namespaces import Namespaces


class AlbumClassification():
    '''
    Subclass for handling catalog data specific
    to a music album.
    '''
    def __init__(self, in_element):
        super().__init__()
        self.genres = None
        self.soundtrack = None
        self.score = None
        if in_element is not None:
            self._process(in_element)

    def _process(self, in_element):
        for child in (in_element):
            e_name = Namespaces.ns_strip(child.tag)
            if e_name == 'genres':
                self.genres = AlbumClassificationGenres(child)
            elif e_name == 'soundtrack':
                self.soundtrack = AlbumClassificationSoundtrack()
            elif e_name == 'score':
                self.score = AlbumClassificationSoundtrack()


class AlbumClassificationGenres():
    '''
    Album genres.
    '''
    def __init__(self, in_element):
        self.primary = ''
        self.secondary = []
        self.subgenres = []
        if in_element is not None:
            self._process(in_element)

    def _process(self, in_element):
        for child in in_element:
            e_name = Namespaces.ns_strip(child.tag)
            if e_name == 'primary':
                self.primary = child.text
            if e_name == 'secondary':
                self.secondary.append(child.text)
            if e_name == 'subgenres':
                self._process_subgenres(child)

    def _process_subgenres(self, in_element):
        for child in in_element:
            e_name = Namespaces.ns_strip(child.tag)
            if e_name == 'subgenre':
                self.subgenres.append(in_element.text)


class AlbumClassificationSoundtrack():
    '''
    Optional element for albums that are soundtracks or scores.
    '''
    def __init__(self):
        self.content = []



#!/usr/bin/env python

#
# Copyright 2025 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''Code suitable for ad-hoc inspections of a media object.'''

import random
from media.data.media import Media
from media.data.media.contents.movie import Movie
from media.data.media.contents.audio.album import Album


class Inspector():
    '''
    A class to provide short details on a media object.
    '''
    def __init__(self):
        pass

    def inspect(self, in_media):
        '''
        Return information about a media object.
        '''
        out = ''
        if not issubclass(in_media.__class__, Media):
            return 'Not a media object.'
        out += "=" * 35 + "\n"
        out += "Media object confirmed.\n"
        out += f"Media Title: {in_media.title!s}\n"
        out += self._inspect_library(in_media)
        out += self._inspect_contents(in_media)
        out += "=" * 35 + "\n"
        return out

    def _inspect_library(self, in_media):
        out = ''
        if in_media.library:
            if len(in_media.library.instances) > 0:
                out += f"Instance count: {len(in_media.library.instances)}\n"
            else:
                out += "No instances.\n"
        return out

    def _inspect_contents(self, in_media):
        out = ''
        if len(in_media.contents) > 0:
            out += f"Total content objects: {len(in_media.contents)}\n"
            for con in in_media.contents:
                if issubclass(con.__class__, Movie):
                    out += self._inspect_con_movie(con)
                elif issubclass(con.__class__, Album):
                    out += self._inspect_con_album(con)
        return out

    def _inspect_con_movie(self, in_con):
        out = "\nMovie Object Found\n"
        if in_con.title:
            out += f"    Title: {in_con.title}\n"
        if in_con.crew:
            if in_con.crew.cast.cast:
                act_count = len(in_con.crew.cast.cast)
                out += f"    Actor Count: {act_count}\n"
                out += f"    First Actor: {in_con.crew.cast.cast[0].actor}\n"
        return out

    def _inspect_con_album(self, in_con):
        out = "Album Object Found\n"
        art_str = ""
        if in_con.title:
            out += f"    Title: {in_con.title}\n"
        if in_con.catalog:
            if in_con.catalog.artists:
                for art_i in in_con.catalog.artists:
                    art_str += str(art_i) + " "
        if art_str:
            out += f"    Artists: {art_str}\n"
        out += f"    Element Count: {len(in_con.elements)}\n"
        return out

    @classmethod
    def rand_media(cls, in_matches):
        '''
        Return a random media object from a small list.
        '''
        r_num = random.randint(0, len(in_matches)-1)
        return in_matches[r_num]

    def find_random_album(self, in_repo):
        '''
        Find a media object that contains at least
        one album.
        '''
        matches = []
        for media in in_repo.media:
            if len(media.contents) > 0:
                for medcon in media.contents:
                    if issubclass(medcon.__class__, Album):
                        matches.append(media)
                        break
        return Inspector.rand_media(matches)

    def find_random_movie(self, in_repo):
        '''
        Find a media object that contains at least
        one movie.
        '''
        matches = []
        for media in in_repo.media:
            if len(media.contents) > 0:
                for medcon in media.contents:
                    if issubclass(medcon.__class__, Movie):
                        matches.append(media)
                        break
        return Inspector.rand_media(matches)



#!/usr/bin/env python

#
# Copyright 2025 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Classes related to cast members and the roles they portray.
'''

# pylint: disable=too-few-public-methods
# pylint: disable=R0801
# pylint: disable=too-many-instance-attributes

import xml.etree.ElementTree as ET
from media.xml.functions import xs_bool
from media.xml.namespaces import Namespaces
from media.data.nouns import PersonalName
from media.data.media.contents.generic.story.characters import CharacterName
from media.general.stringtools import build_sort_string


class Cast():
    '''
    Container object for all cast members, or a subset of cast members.
    '''
    def __init__(self, in_element):
        self.cast = []
        if in_element is not None:
            role_count = 1
            for child in in_element:
                tagname = Namespaces.ns_strip(child.tag)
                if tagname == 'role':
                    self.cast.append(Role(child, role_count))
                    role_count += 1


class Role():
    '''
    A role contains the name of a single actor, and one or
    more portrayal objects.
    '''
    def __init__(self, in_element, in_count):
        self.actor = None
        self.portrays = []
        self.order = in_count
        if in_element is not None:
            for child in in_element:
                tagname = Namespaces.ns_strip(child.tag)
                if tagname == 'actor':
                    self.actor = Actor(child)
                elif tagname == 'self':
                    self.portrays.append(PortraysSelf())
                elif tagname == 'narrator':
                    self.portrays.append(PortraysNarrator())
                elif tagname == 'background':
                    self.portrays.append(PortraysBackground())
                elif tagname == 'additionalVoices':
                    self.portrays.append(PortraysAdditionalVoices())
                elif tagname == 'character':
                    chr_obj = Role.character_dispatcher(child, self.actor)
                    self.portrays.append(chr_obj)

    @classmethod
    def character_dispatcher(cls, in_element, in_actor):
        '''
        Returns a Portrays element based on the inner
        XML structure.
        '''
        tagname = Namespaces.ns_strip(in_element[0].tag)
        if tagname == 'name':
            return PortraysNamedCharacter(in_element)
        if tagname == 'unnamed':
            return PortraysUnnamedCharacter(in_element)
        if tagname == 'self':
            return PortraysSelfCharacter(in_actor)
        return None

    def __str__(self):
        return f"Role object for actor: {str(self.actor)}"


class AbstractActor(PersonalName):
    '''
    Abstract root class for an actor.
    '''


class Actor(AbstractActor):
    '''
    Actor class.
    '''
    def __init__(self, in_element):
        super().__init__(in_element)
        self.archival_footage = False
        if 'archivalFootage' in in_element.attrib:
            if xs_bool(in_element.attrib['archivalFootage']):
                self.archival_footage = True


class AbstractPortrays():
    '''
    Abstract root class for all portrays objects.

    '''
    def __init__(self):
        self.value = ''
        self.formal = ''
        self.sort_value = ''

    def __str__(self):
        return self.value

    def __hash__(self):
        '''
        Hash values are only guaraunteed to be unique
        against other values against the same actor within
        the same role structure.
        '''
        return hash(self.formal)

    def __lt__(self, other):
        return self.sort_value < other.sort_value

    def __rt__(self, other):
        return self.sort_value > other.sort_value

    def __eq__(self, other):
        return self.sort_value == other.sort_value


class PortraysNarrator(AbstractPortrays):
    '''
    Class designating the special role Narrator
    '''
    def __init__(self):
        super().__init__()
        self.value = 'Narrator'
        self.formal = 'Narrator'
        self.sort_value = 'narrator'

    def to_element(self):
        '''
        Generate an empty additionalVoices element.
        '''
        return ET.Element('narrator')


class PortraysSelf(AbstractPortrays):
    '''
    Class for an actor as themselves.
    '''
    def __init__(self):
        super().__init__()
        self.value = 'Self'
        self.formal = 'Self'
        self.sort_value = 'self'


class PortraysBackground(AbstractPortrays):
    '''
    Class for special background character role.
    '''
    def __init__(self):
        super().__init__()
        self.value = 'Background'
        self.formal = 'Background'
        self.sort_value = 'background'

    def to_element(self):
        '''
        Generate an empty background element.
        '''
        return ET.Element('background')


class PortraysAdditionalVoices(AbstractPortrays):
    '''
    Class for special additionalVoices character role.
    '''
    def __init__(self):
        super().__init__()
        self.value = 'Additional Voices'
        self.formal = 'Additional Voices'
        self.sort_value = 'additional voices'


class AbstractPortraysCharacter(AbstractPortrays):
    '''
    Abtract root class of all Portrays objects.
    '''


class PortraysNamedCharacter(AbstractPortraysCharacter):
    '''
    A character with a partial or complete name.
    '''
    def __init__(self, in_element):
        super().__init__()
        self.name = None
        self.aliases = []
        self.variant = None
        self.aspect = None
        self._process(in_element)

    def _process(self, in_element):
        for child in in_element:
            tagname = Namespaces.ns_strip(child.tag)
            if tagname == 'name':
                self.name = CharacterName(child)
            elif tagname == 'alias':
                self.aliases.append(CharacterName(child))
            elif tagname == 'variant':
                self.variant = child.text
            elif tagname == 'aspect':
                self.aspect = child.text
        self.sort_value = self.name.sort_value
        self._build_formal_name()

    def _build_formal_name(self):
        self.formal = self.name.formal
        if self.variant:
            self.formal += f" ({self.variant})"
        if self.aspect:
            self.formal += f" ({self.aspect})"
        if len(self.aliases) > 0:
            for alias_name in self.aliases:
                self.formal += ' / ' + alias_name.formal


class PortraysSelfCharacter(AbstractPortraysCharacter):
    '''
    Portrayal of a character version of themselves.
    '''
    def __init__(self, in_actor):
        super().__init__()
        self.name = in_actor
        self.sort_value = self.name.sort_value
        self.formal = str(self.name)


class PortraysUnnamedCharacter(AbstractPortraysCharacter):
    '''
    A character that exists, but doesn't have a
    complete or partial name.
    '''
    def __init__(self, in_element):
        super().__init__()
        self.unnamed = ''
        self.variant = None
        self.aspect = None
        self._process(in_element)

    def _process(self, in_element):
        for child in in_element:
            tagname = Namespaces.ns_strip(child.tag)
            if tagname == 'unnamed':
                self.unnamed = child.text
                self.formal = child.text
                self.sort_value = build_sort_string(child.text)
            elif tagname == 'variant':
                self.variant = child.text
            elif tagname == 'aspect':
                self.aspect = child.text



#!/usr/bin/env python

#
# Copyright 2025 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''Common code related to manipulating titles.'''

from media.general.stringtools import transform_string, build_sort_string
from media.general.language import LanguageHelpers


class TitleMunger():
    '''
    Munge title strings into usable data

    Works off of a passed title object, not a raw string.
    '''

    @classmethod
    def build_filename_string(cls, in_title):
        '''
        Build a string suitable for a filename.

        Ex: The Ugly Brige 2 becomes the_ugly_bridge_2
        '''
        level1 = transform_string(str(in_title))
        return level1.translate(level1.maketrans(" \t\n\r", "____"))

    @classmethod
    def build_sort_title_string(cls, in_title):
        '''
        Create a sortable string based on the title.

        Ex: "The Ugly Bridge 2" becomes "ugly_bridge_2_+the"
        '''
        return build_sort_string(str(in_title))

    @classmethod
    def _chk_cat_sort_title(cls, in_catalog):
        if in_catalog:
            if in_catalog.alt_titles:
                if in_catalog.alt_titles.variant_sort is True:
                    return str(in_catalog.alt_titles.variant_title)
        return None

    @classmethod
    def build_sort_cat_title(cls, in_title, in_catalog):
        '''
        Create a sortable string based in the title
        and catalog info.

        Ex: "9 Ugly Bridges" becomes "nine_ugly_bridges"
        '''
        working = TitleMunger._chk_cat_sort_title(in_catalog) or str(in_title)
        return build_sort_string(working)

    @classmethod
    def build_unique_key_string(cls, in_title, in_catalog):
        '''
        Creates a unique string for the content based on
        title and catalog info.

        Ex: "The Ugly Bridge 2" becomes "ugly_bridge_2_+the-2023-01"
        '''
        level1 = TitleMunger.build_sort_cat_title(str(in_title), in_catalog)
        year = '0000'
        idx = '1'
        if in_catalog:
            if in_catalog.copyright:
                if in_catalog.copyright.year:
                    year = str(in_catalog.copyright.year)
            if in_catalog.unique_index:
                idx = f"{in_catalog.unique_index.index:d}"
        return level1 + '-' + year + '-' + idx

    @classmethod
    def build_title_path(cls, in_title):
        '''
        Create a path string based on a title,

        Ex: "The 3 Ugly Bridges" becomes "3/3us"
        '''
        level1 = transform_string(str(in_title))
        word_split = level1.split()
        if word_split[0] in LanguageHelpers.Articles_English:
            word_split.pop(0)
        first_letter = word_split[0][0]
        second_letter = word_split[0][1]
        last_letter = word_split[-1][-1]
        path_string = f"/{first_letter}/" + \
                      f"{first_letter}{second_letter}{last_letter}/"
        return path_string

    @classmethod
    def build_catalog_title(cls, in_title, in_catalog):
        '''
        Creates a title value suitable for reference notation.

        Ex: "The Ugly Bridge 2" becomes "The Ugly Bridge 2 (2023)"
        '''
        year = '0000'
        idx = 1
        if in_catalog:
            if in_catalog.copyright:
                if in_catalog.copyright.year:
                    year = str(in_catalog.copyright.year)
            if in_catalog.unique_index:
                idx = in_catalog.unique_index.index
        if idx == 1:
            return f"{in_title!s} ({year})"
        return f"{in_title!s} ({year}/{idx})"



#!/usr/bin/env python

#
# Copyright 2025 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''Classes and subroutines for media display.'''

from media.data.media.medium.device import MediumDeviceMap
from media.data.media.medium.release import FormalType
from media.fmt.text.basics import (hdr_list,
                                   hdr_list_oneper)
from media.fmt.text.movie import MiniEntry as MovieEntry


class ListEntry():
    '''
    An entry for a piece of physical media.
    '''
    def __init__(self, in_media):
        self.media = in_media
        self.type = ''
        self.type_new = False
        self.date = ''
        self.copies = 0
        self.title_key = ''
        self._prep_fields()
        if self.type:
            if self.type_new:
                f_type = MediumDeviceMap.formal_convert(self.type)
            else:
                f_type = FormalType.formal_convert(self.type)
        else:
            f_type = 'UNKNOWN'
        self.output = f"{self.media.title!s:45.45s} {f_type:10s} " + \
                      f"{self.copies:6d} {self.date!s:s}"

    def _prep_fields(self):
        if self.media.medium.device:
            self.type = self.media.medium.device.type_name
            self.type_new = True
        elif self.media.medium.release:
            if self.media.medium.release.type:
                self.type = self.media.medium.release.type
        if self.media.library:
            if self.media.library.instances:
                self.copies = len(self.media.library.instances)
                if self.media.library.instances[0].acquisition:
                    acq = self.media.library.instances[0].acquisition
                    self.date = acq.date
        self.title_key = self.media.title.sort_title

    def __str__(self):
        return self.output

    @classmethod
    def header(cls):
        '''
        Output a report header.
        '''
        output = f"{'Title':45s} {'Media Type':10s} {'Copies':6s} " + \
                 f"{'Date':19s}\n" + \
                 f"{'-' * 45} {'-' * 10} {'-' * 6} {'-' * 19}"
        return output


class BriefEntry():
    '''Formatting for a brief text record of a media device'''
    def __init__(self, in_media):
        self.media = in_media
        self._prep_fields()
        self._build_output()

    def _prep_fields(self):
        self.title_key = self.media.title.sort_title
        if self.media.medium:
            medm = self.media.medium
            if medm.device:
                self.f_type = MediumDeviceMap.formal_convert(
                        medm.device.type_name)
            elif medm.release:
                self.f_type = FormalType.formal_convert(medm.release.type)
            else:
                self.f_type = 'UNKNOWN'

    def _build_output(self):
        self.output = self.entry_header()
        self.output += self.library_instances_rep()
        self.output += self.library_filing_rep()
        self.output += self.prod_id_rec()
        self.output += self.specs_inventory_rep()
        self.output += self.contents_rep()
        self.output += "\n\n"

    def entry_header(self):
        '''
        Header line for the entry.
        '''
        out = f"{self.media.title!s:60s} {self.f_type:>8s}\n" + \
              f"{'=' * 69}\n"
        return out

    def prod_id_rec(self):
        '''Product identification information.'''
        out = ''
        if self.media.medium.product_id:
            out += "Product Id > Codes\n\n"
            ids = self.media.medium.product_id
            if ids.barcodes:
                out += hdr_list_oneper('Barcode',
                                       ids.barcodes,
                                       indent=2,
                                       hformat=">8s")
            if ids.skus:
                out += hdr_list_oneper('SKU',
                                       ids.skus,
                                       indent=2,
                                       hformat=">8")
            out += "\n"
        return out

    def library_instances_rep(self, indent=2):
        '''Library information regarding copies of media.'''
        out = ''
        if self.media.library:
            lib = self.media.library
            if lib.instances:
                out += "Library > Instances\n\n"
                out += f"{' ' * indent}{'ID Label'} {'Acquisition':30s} " + \
                       f"{'Date':19s}\n" + \
                       f"{' ' * indent}{'-' * 8} {'-' * 30} {'-' * 19}\n"
                for inst_i in lib.instances:
                    if not inst_i.local_id:
                        id_val = 'UNDEF'
                    else:
                        id_val = inst_i.local_id
                    if not inst_i.acquisition:
                        acq_d = 'UNKNOWN DATE'
                    else:
                        acq_d = inst_i.acquisition.date
                    out += f"{' ' * indent}{id_val:8s} " + \
                           f"{inst_i.acquisition!s:30s} " + \
                           f"{acq_d!s:19s}\n"
                out += "\n"
        return out

    def library_filing_rep(self):
        '''Library information regarding filing.'''
        out = ''
        if self.media.library:
            lib = self.media.library
            if lib.filing:
                out += "Library > Filing\n\n"
                if lib.filing.catalog:
                    fcat = lib.filing.catalog
                    out += f"    {'Catalog'}: {fcat}\n"
                if len(lib.filing.collections) > 0:
                    out += hdr_list('    Collection',
                                    lib.filing.collections)
                out += "\n\n"
        return out

    def specs_inventory_rep(self):
        '''Product Specs inventory report.'''
        out = ''
        if self.media.medium.product_specs:
            out += "Product Specs > Inventory\n\n"
            inv = self.media.medium.product_specs.inventory.inventory
            out += media_container_list(inv, padding=2)
            out += "\n"
        return out

    def contents_rep(self):
        '''Report on the contents within the media object.'''
        out = "Contents > Movies\n\n"
        out += MovieEntry.header_line()
        for con in self.media.contents:
            entry = MovieEntry(con)
            out += f"{entry!s}"
        return out

    def __str__(self):
        return self.output


def media_container_list(in_containers, padding=0):
    '''
    Take an array of containers, and then output the
    objects within them.
    '''
    padding_s = f"{' ' * padding}"
    out = padding_s
    for m_item in in_containers:
        out += media_container_object(m_item, padding=padding)
    return out


def media_container_object(in_container, padding=0, indent=0):
    '''
    Output the individual elements of a container.

    If the container contains containers, recursively
    call the function against the found container.
    '''
    padding_s = f"{' ' * padding}"
    indent_s = f"{' ' * indent}"
    label = str(in_container) + ' > '
    out = label
    subcontainers = in_container.subcontainers_list()
    media_s = in_container.media_string_count_list()
    for m_item in subcontainers:
        out += media_container_object(m_item,
                                      padding=padding,
                                      indent=len(label))
    m_count = 1
    for m_item in media_s:
        out += m_item + "\n"
        if m_count < len(media_s):
            out += f"{padding_s}{indent_s}{' ' * len(label)}"
        else:
            out += f"{padding_s}{indent_s}"
        m_count += 1
    return out



#!/usr/bin/env python

#
# Copyright 2024 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Song object structure
'''

# pylint: disable=too-few-public-methods
# pylint: disable=R0801

from media.data.media.contents.genericv.technical import process_iso_duration
from media.data.media.contents.audio.elements import (
        AbstractElement, ElementCatalog, ElementTechnical
        )
from media.data.nouns import noun_dispatcher
from media.xml.namespaces import Namespaces


class Song(AbstractElement):
    '''
    Object representation of a song.
    '''
    def __init__(self, in_element, in_parent):
        super().__init__()
        self.catalog = None
        self.classification = None
        self.technical = None
        self.parent_o = in_parent
        self._process(in_element)

    def _process(self, in_element=None):
        super()._process(in_element)
        for child in in_element:
            e_name = Namespaces.ns_strip(child.tag)
            if e_name == 'technical':
                self.technical = SongTechnical(child)
            elif e_name == 'catalog':
                self.catalog = SongCatalog(child)
        self._post_load_process()

    def _post_load_process(self):
        if not self.catalog:
            self._extract_catalog_from(self.parent_o)

    def _extract_catalog_from(self, in_parent):
        super()._extract_catalog_from(in_parent)
        if in_parent.catalog is not None:
            if in_parent.catalog.composers is not None:
                self.catalog.composers = in_parent.catalog.composers


class SongCatalog(ElementCatalog):
    '''
    Catalog specific to a song.
    '''
    def __init__(self, in_element):
        super().__init__(in_element)
        self.composers = []
        if in_element is not None:
            self._process(in_element)

    def _process(self, in_element):
        super()._process(in_element)
        for child in in_element:
            e_name = Namespaces.ns_strip(child.tag)
            if e_name == 'composers':
                self._process_composers(child)

    def _process_composers(self, in_element):
        for child in in_element:
            self.composers.append(SongComposer(child))


class SongComposer():
    '''
    Object representing a composer.
    '''
    def __init__(self, in_element):
        self.name = None
        self.publishers = []
        self.rights = None
        if in_element is not None:
            self._process(in_element)

    def _process(self, in_element):
        for child in in_element:
            e_name = Namespaces.ns_strip(child.tag)
            if e_name == 'name':
                self.name = noun_dispatcher(child)
            elif e_name == 'publisher':
                self.publishers.append(child.text)
            elif e_name == 'rights':
                self.rights = child.text


class SongTechnical(ElementTechnical):
    '''
    Object representation of a song's technical data.
    '''
    def __init__(self, in_element):
        super().__init__(in_element)
        self.recording = None
        if in_element is not None:
            self._process(in_element)

    def _process(self, in_element):
        super()._process(in_element)
        for child in in_element:
            e_name = Namespaces.ns_strip(child.tag)
            if e_name == 'studioRecording':
                self.recording = SongTechnicalRecording.STUDIO
            elif e_name == 'liveRecording':
                self.recording = SongTechnicalRecording.LIVE
            elif e_name == 'demoRecording':
                self.recording = SongTechnicalRecording.DEMO
            elif e_name == 'runtime':
                self.runtime = SongTechnicalRuntime(child)
            elif e_name == 'tempo':
                self.tempo = SongTechnicalTempo(child)


class SongTechnicalRecording():
    '''
    Identifying the type of recording for a song.
    '''
    STUDIO = 1
    LIVE = 2
    DEMO = 3

    @classmethod
    def to_string(cls, in_recording):
        '''
        Return a suitable string value.
        '''
        matrix = {
                SongTechnicalRecording.STUDIO: 'Studio Recording',
                SongTechnicalRecording.LIVE: 'Live Recording',
                SongTechnicalRecording.DEMO: 'Demo Recording'
        }
        if in_recording in matrix:
            return matrix[in_recording]
        return None


class SongTechnicalRuntime():
    '''
    Song runtime data.
    '''
    def __init__(self, in_element):
        self.overall = None
        if in_element is not None:
            self._process(in_element)

    def _process(self, in_element):
        for child in in_element:
            e_name = Namespaces.ns_strip(child.tag)
            if e_name == 'overall':
                dur_string = child.text
                dur_value = process_iso_duration(dur_string)
                self.overall = dur_value


class SongTechnicalTempo():
    '''
    Song tempo or bpm data.
    '''
    def __init__(self, in_element):
        self.bpm = 0
        if in_element is not None:
            self._process(in_element)

    def _process(self, in_element):
        for child in in_element:
            e_name = Namespaces.ns_strip(child.tag)
            if e_name == 'bpm':
                self.bpm = int(child.text)



#!/usr/bin/env python

#
# Copyright 2023 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Classes related to crewmembers of a piece
of visual art, like a movie or television show.
'''

# pylint: disable=too-few-public-methods
# pylint: disable=R0801
# pylint: disable=too-many-instance-attributes

from media.xml.namespaces import Namespaces
from media.data.nouns import PersonalName, noun_dispatcher
from media.data.media.contents.genericv.crew.cast import Cast


class Crew():
    '''
    Main container element for all crew objects.
    '''
    def __init__(self, in_element):
        self.directors = []
        self.editors = []
        self.writers = []
        self.cinemap = []
        self.music = None
        self.cast = None
        if in_element is not None:
            self._process(in_element)

    def _process(self, in_element):
        for child in in_element:
            tagname = Namespaces.ns_strip(child.tag)
            if tagname == 'directors':
                self.directors = Directors.load(child)
            elif tagname == 'editors':
                self.editors = SimpleCrew.load(child, 'editor')
            elif tagname == 'cinemaphotographers':
                self.cinemap = SimpleCrew.load(child, 'cinemaphotographer')
            elif tagname == 'writers':
                self.writers = SimpleCrew.load(child, 'writer')
            elif tagname == 'music':
                self.music = Music(child)
            elif tagname == 'cast':
                self.cast = Cast(child)


class Directors():
    '''
    Simple class method to parse director elements.
    '''
    @classmethod
    def load(cls, in_element):
        ''' Load an array of names for a given element in the XML'''
        out = []
        if in_element is not None:
            for child in in_element:
                tagname = Namespaces.ns_strip(child.tag)
                if tagname == 'director':
                    out.append(PersonalName(child))
        return out


class Writers():
    '''
    Class method to parser writer elements, which
    are a little more complex compared to other
    crew elements.
    '''
    @classmethod
    def load(cls, in_element):
        ''' Load an array of writers based on the XML'''
        out = []
        if in_element is not None:
            for child in in_element:
                tagname = Namespaces.ns_strip(child.tag)
                if tagname == 'writer':
                    out.append(PersonalName(child))


class SimpleCrew():
    '''
    Generic crew loader object, which returns
    a list of Name elements.
    '''
    @classmethod
    def load(cls, in_element, desired_tag):
        '''Load an array of names for a given element in the XML'''
        out = []
        if in_element is not None:
            for child in in_element:
                tagname = Namespaces.ns_strip(child.tag)
                if tagname == desired_tag:
                    out.append(PersonalName(child))
        return out

    @classmethod
    def output(cls, title, in_list):
        '''Special output formatter to string multiple names together'''
        out_string = ""
        if len(in_list) == 1:
            out_string = f"{title}: {in_list[0]}"
        else:
            out_string = f"{title}"
            for name in in_list:
                out_string += f"{name}. "
            out_string = out_string[:-2]
        return out_string


class CrewAssignment():
    '''
    An assignment to a crew position.
    '''
    def __init__(self, tagname, in_element):
        self.title = tagname
        self.uncredited = False
        self.object = noun_dispatcher(in_element)
        if 'uncredited' in in_element.attrib:
            self.uncredited = True

    def __str__(self):
        return str(self.object)


class Music():
    '''
    Container object for all music related staff.
    '''
    def __init__(self, in_element):
        self.allowed = ['composer', 'conductor', 'music']
        self.breakdown = {}
        self.composers = []
        self.conductors = []
        self.music = []
        self._process(in_element)

    def _process(self, in_element):
        for child in in_element:
            tagname = Namespaces.ns_strip(child.tag)
            if tagname in self.allowed:
                # nx_element = child[0]
                crew_obj = CrewAssignment(tagname, child)
                if tagname == 'composer':
                    self.composers.append(crew_obj)
                elif tagname == 'music':
                    self.music.append(crew_obj)
                elif tagname == 'conductor':
                    self.conductors.append(crew_obj)



#!/usr/bin/env python

#
# Copyright 2025 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
All object classes to store test status information.
'''

# pylint: disable=R0903

from media.data.media.medium import get_medium_type


class StatusCode():
    '''
    Simple status code.
    '''
    PASS = 0
    FAIL = 1

    @classmethod
    def to_str(cls, in_code):
        '''
        Return the status code as a string value.
        '''
        matrix = {StatusCode.PASS: 'Pass',
                  StatusCode.FAIL: 'Fail'}
        if in_code in matrix:
            return matrix[in_code]
        raise ValueError('Invalid Status Code')


class Tray():
    '''
    An object containing both a piece of media
    and requested testing parameters.
    '''
    def __init__(self, in_media, in_level):
        self.media = in_media
        self.level = in_level
        self.status = MediaStatus(in_media, in_level)

    def has_passed(self):
        '''
        Boolean identifier if a piece of media
        has passed all tests.
        '''
        return self.status.has_passed()


class AbstractStatus():
    '''
    An abstract status object
    '''
    def __init__(self, in_level):
        self.level = in_level

    @classmethod
    def rep_test_header(cls):
        '''
        Generate a header for a list of individual
        test results
        '''
        out = f"\n{'No':4s} {'Test':40s} {'Result':7s} " + \
              f"{'Score':5s} {'Max':5s} {'Faults':6s}\n" + \
              f"{'-'*4} {'-'*40} {'-'*7} {'-'*5} {'-'*5} {'-'*6}\n"
        return out

    @classmethod
    def rep_fault_header(cls):
        '''
        Generate a header for a list of fault
        details
        '''
        out = f"\n{' '*25}<<< FAULTS >>>\n\n" + \
              f"{'Test':40s} {'Level':10s} {'Message':s}\n" + \
              f"{'-'*40} {'-'*10} {'-'*20}\n"
        return out


class MediaStatus(AbstractStatus):
    '''
    A status object for a piece of media.
    '''
    def __init__(self, in_media, in_level):
        super().__init__(in_level)
        self.media = in_media
        self.contents = []
        self.test_results = []
        self.tally = Tally()
        self.faults = []
        self.setup()

    def setup(self):
        '''
        Set up ContentStatus objects for every piece of content.
        '''
        for media_con in self.media.contents:
            self.contents.append(ContentStatus(media_con, self.level))

    def add_result(self, in_test_result):
        '''
        Add a new test result object to the array, and
        update the total score tally information.
        '''
        self.tally.add_result_tally(in_test_result)
        self.test_results.append(in_test_result)

    def calculate(self):
        '''
        Combine the tally scores for all content objects.
        '''
        for con in self.contents:
            self.tally.merge_tally_object(con.tally)

    def has_passed(self):
        '''
        Boolean identifier if a piece of media has passed all tests.
        '''
        return self.tally.has_passed()

    def report(self):
        '''
        Generate a full report on all test results and the
        total score.
        '''
        out = ''
        out += self._report_header()
        out += self._report_main_summary()
        out += self._report_main_body()
        out += self._report_content_objects()
        return out

    def _report_header(self):
        ftype = get_medium_type(self.media.medium)
        out = f"\n{'='*72}\n" + \
              f"{self.media.title!s:61s} {ftype:>10s}\n" + \
              f"{'='*72}\n"
        return out

    def _report_main_summary(self):
        out = f"\n{'Score':>32s} : {self.tally.score_str():s}\n" + \
              f"{'Tests Run':>32s} : {self.tally.run:d}\n" + \
              f"{'Tests Skipped':>32s} : {self.tally.skipped:d}\n" + \
              f"{'Passed':>32s} : {self.tally.passed:d}\n" + \
              f"{'Failed':>32s} : {self.tally.failed:d}\n\n"
        return out

    def _report_main_body(self):
        out = ''
        out += MediaStatus.rep_test_header()
        top_level_faults = 0
        line_count = 1
        for test_result in self.test_results:
            out += f"{line_count:4d} {test_result}\n"
            line_count += 1
            if len(test_result.faults) > 0:
                top_level_faults += len(test_result.faults)
        if top_level_faults > 0:
            out += MediaStatus.rep_fault_header()
            for test_result in self.test_results:
                fault_count = 1
                for fault in test_result.faults:
                    if fault_count < 2:
                        out += f"{test_result.name:40s} " + \
                               f"{fault.lvl_str():10s} " + \
                               f"{fault.message}\n"
                    else:
                        out += f"{' ':40s} " + \
                               f"{fault.lvl_str():10s} " + \
                               f"{fault.message}\n"
                    fault_count += 1
        return out

    def _report_content_objects(self):
        out = ''
        for con_status in self.contents:
            line_count = 1
            out += f"\nContent: {con_status.item.title!s:55} {'(Movie)'}\n" + \
                   f"{'-'*72}\n" + \
                   MediaStatus.rep_test_header()
            cfc = 0
            for test_result in con_status.test_results:
                out += f"{line_count:4d} {test_result}\n"
                line_count += 1
                cfc += len(test_result.faults)
            if cfc > 0:
                out += MediaStatus.rep_fault_header()
                for test_result in con_status.test_results:
                    fault_count = 1
                    for fault in test_result.faults:
                        if fault_count < 2:
                            out += f"{test_result.name:40s} " + \
                                   f"{fault.lvl_str():10s} " + \
                                   f"{fault.message}\n"
                        else:
                            out += f"{' ':40s} " + \
                                   f"{fault.lvl_str():10s} " + \
                                   f"{fault.message}\n"
        return out


class ContentStatus(AbstractStatus):
    '''
    A status object for content in a piece
    of media.
    '''
    def __init__(self, in_content, in_level):
        super().__init__(in_level)
        self.item = in_content
        self.test_results = []
        self.tally = Tally()
        # self.fc = 0
        self.possible = 0
        self.actual = 0

    def add_result(self, in_test_result):
        '''
        Add a new test result object to the array
        and update the score totals.
        '''
        self.tally.add_result_tally(in_test_result)
        self.test_results.append(in_test_result)

    def calculate(self):
        '''
        Total up the tally results.

        (this method might not be used)
        '''
        for tst in self.test_results:
            if not tst.skipped:
                self.tally.max_points += tst.possible
                # self.possible += tst.possible
                self.tally.earned_points += tst.score
                # self.actual += tst.score
                if len(tst.faults) > 0:
                    self.tally.faults.extend(tst.faults)
                    # self.fc += len(tst.faults)


class Tally():
    '''
    A simple scoring object for tracking
    status scores.
    '''
    def __init__(self):
        self.run = 0
        self.skipped = 0
        self.passed = 0
        self.failed = 0
        self.max_points = 0
        self.earned_points = 0
        self.faults = []

    def add_result_tally(self, in_result):
        '''
        Add the scoring information from a single test result.
        '''
        if in_result.skipped:
            self.skipped += 1
        else:
            self.run += 1
            if in_result.status_code == StatusCode.PASS:
                self.passed += 1
            else:
                self.failed += 1
            self.max_points += in_result.possible
            self.earned_points += in_result.score
            if len(in_result.faults) > 0:
                self.faults.extend(in_result.faults)

    def merge_tally_object(self, in_other):
        '''
        Merge two tally objects together so
        all values are in one object.
        '''
        self.max_points += in_other.max_points
        self.earned_points += in_other.earned_points
        self.run += in_other.run
        self.skipped += in_other.skipped
        self.passed += in_other.passed
        self.failed += in_other.failed
        if len(in_other.faults) > 0:
            self.faults.extend(in_other.faults)

    def rs_str(self):
        '''
        Return a comparision of run vs. skipped tests.
        '''
        return f"{self.run:3d} {self.skipped:4d}"

    def pf_str(self):
        '''
        Return a comparision of passed vs. failed tests.
        '''
        return f"{self.passed:4d} {self.failed:4d}"

    def ratio(self):
        '''
        Return a score value of all passing tests
        as a value not to exceed 10.0.
        '''
        return self.earned_points / self.max_points * 10

    def has_passed(self):
        '''
        Quick result on whether a piece of media has
        passed all tests.
        '''
        return self.earned_points == self.max_points

    @classmethod
    def header(cls):
        '''
        Return a simple header line to match the
        output fields.
        '''
        return f"{'Media':40s} " + \
               f"{'Type':10s} " + \
               f"{'Score':10s} {'Run/Skip':8s} {'Pass/Fail':9s} " + \
               f"{'Faults':6s}\n" + \
               f"{'-'*40} {'-'*10} {'-'*10} {'-'*8} {'-'*9} {'-'*6}\n"

    def score_str(self):
        '''
        Return a string in the format of score value against the
        highest value 10.0
        '''
        return f"{self.ratio():.1f}/10.0"

    def __str__(self):
        '''
        Return score value, test run/skipped,
        tests passed/failed, and the number of faults.
        '''
        return f"{self.score_str():>10s} " + \
               f"{self.rs_str()} " + \
               f"{self.pf_str()} " + \
               f"{len(self.faults):6d}"


class TestResult():
    '''
    Keeps data tied to the result of a single test.
    '''
    def __init__(self, in_name, in_highest):
        self.name = in_name
        self.possible = in_highest
        self.score = 0
        self.skipped = False
        self.status_code = StatusCode.FAIL
        self.faults = []

    def skip(self):
        '''
        Identify a test as skipped, which
        would have no score.
        '''
        self.skipped = True
        self.score = 0
        self.possible = 0

    def result(self, in_score, in_pf, in_fault=None):
        '''
        Set the score value, the test status code,
        and a fault object, if any.
        '''
        if in_score > self.possible:
            raise ValueError('Score value too high.')
        self.score = in_score
        self.status_code = in_pf
        if in_fault:
            self.faults.append(in_fault)

    def autopass(self):
        '''
        Shortcut to automatically set a test as passing.
        '''
        self.score = self.possible
        self.status_code = StatusCode.PASS

    def add_fault(self, in_fault):
        '''
        Add a single fault to the fault array.
        '''
        self.faults.append(in_fault)

    def add_faults(self, in_fault_list):
        '''
        Add multiple faults to the fault array.
        '''
        self.faults.extend(in_fault_list)

    def __str__(self):
        if not self.skipped:
            return f"{self.name:40s} " + \
                   f"{StatusCode.to_str(self.status_code):7s} " + \
                   f"{self.score:5d} " + \
                   f"{self.possible:5d} " + \
                   f"{len(self.faults):6d}"
        return f"{self.name:40s} " + \
               f"{'Skipped':7s} " + \
               f"{'-':>5s} {'-':>5s} {'-':>6s}"
